@@ -197,12 +197,14 @@ class KafkaEventBroker(EventBroker):
         else:
             partition_key = None
 
-        headers = [
-            (
-                "RASA_ENVIRONMENT",
-                bytes(self.rasa_environment, encoding=DEFAULT_ENCODING),
-            )
-        ]
+        headers = []
+        if self.rasa_environment:
+            headers = [
+                (
+                    "RASA_ENVIRONMENT",
+                    bytes(self.rasa_environment, encoding=DEFAULT_ENCODING),
+                )
+            ]
 
         logger.debug(
             f"Calling kafka send({self.topic}, value={event},"
