@@ -1,7 +1,6 @@
 import abc
 import json
 from collections import OrderedDict
-import operator
 from pathlib import Path
 
 import rasa.shared.nlu.training_data.util
@@ -19,7 +18,8 @@ from rasa.shared.nlu.constants import (
 
 import rasa.shared.utils.io
 import typing
-from typing import Text, Dict, Any, Union
+from collections import defaultdict
+from typing import List, Text, Dict, Any, Union
 
 if typing.TYPE_CHECKING:
     from rasa.shared.nlu.training_data.training_data import TrainingData
@@ -43,7 +43,30 @@ class TrainingDataReader(abc.ABC):
         raise NotImplementedError
 
 
+def _raise_on_same_start_and_different_end_positions(
+    aggregated_entities: Dict[int, List[Dict[Text, Any]]],
+) -> None:
+    """Raises a ValueError iff two entities have overlapping but not identical spans.
+
+    Args:
+        aggregated_entities: Entities for each start position
+    """
+    for entity_list in aggregated_entities.values():
+        end = entity_list[0]["end"]
+        for entity in entity_list[1:]:
+            # By construction, start positions of all entities in `entity_list` are
+            # identical
+            if entity["end"] != end:
+                raise ValueError(
+                    f"Entities '{entity}' and "
+                    f"'{aggregated_entities[entity['start']][-1]}' have identical "
+                    f"start but different end positions"
+                )
+
+
 class TrainingDataWriter:
+    """Base class for training data writiers."""
+
     def dump(self, filename: Text, training_data: "TrainingData") -> None:
         """Writes a TrainingData object in markdown format to a file."""
         s = self.dumps(training_data)
@@ -79,8 +102,15 @@ class TrainingDataWriter:
 
     @staticmethod
     def generate_message(message: Dict[Text, Any]) -> Text:
-        """Generates text for a message object."""
+        """Generates text for a message object.
 
+        Args:
+            message: A message
+
+        Returns:
+            The text of the message, annotated with the entity data that is contained
+            in the message
+        """
         md = ""
         text = message.get("text", "")
 
@@ -96,26 +126,41 @@ class TrainingDataWriter:
             entities_with_start_and_end = [
                 e for e in entities if "start" in e and "end" in e
             ]
-            sorted_entities = sorted(
-                entities_with_start_and_end, key=operator.itemgetter("start")
-            )
 
-            for entity in sorted_entities:
-                md += text[pos : entity["start"]]
-                md += TrainingDataWriter.generate_entity(text, entity)
-                pos = entity["end"]
+            # Multiple entities can share the same position span. To account for that,
+            # group all entities by their start positions
+            aggregated_entities: Dict[int, List[Dict[Text, Any]]] = defaultdict(list)
+            for entity in entities_with_start_and_end:
+                aggregated_entities[entity["start"]].append(entity)
+
+            _raise_on_same_start_and_different_end_positions(aggregated_entities)
+
+            for start, entities in sorted(aggregated_entities.items()):
+                md += text[pos:start]
+                md += TrainingDataWriter.generate_entity(
+                    text, entities[0] if len(entities) == 1 else entities,
+                )
+                pos = entities[0]["end"]
 
         md += text[pos:]
 
         return md
 
     @staticmethod
-    def generate_entity(text: Text, entity: Dict[Text, Any]) -> Text:
-        """Generates text for an entity object."""
+    def generate_entity_attributes(
+        text: Text, entity: Dict[Text, Any], short_allowed: bool = True
+    ) -> Text:
+        """Generates text for the entity attributes.
 
-        entity_text = text[
-            entity[ENTITY_ATTRIBUTE_START] : entity[ENTITY_ATTRIBUTE_END]
-        ]
+        Args:
+            text: The text that is annotated with the entity
+            entity: Entity data
+            short_allowed: If `True`, allow shorthand annotation with parenthesis
+
+        Returns:
+            The annotation text that should follow the given text
+        """
+        entity_text = text
         entity_type = entity.get(ENTITY_ATTRIBUTE_TYPE)
         entity_value = entity.get(ENTITY_ATTRIBUTE_VALUE)
         entity_role = entity.get(ENTITY_ATTRIBUTE_ROLE)
@@ -125,11 +170,14 @@ class TrainingDataWriter:
             entity_value = None
 
         use_short_syntax = (
-            entity_value is None and entity_role is None and entity_group is None
+            short_allowed
+            and entity_value is None
+            and entity_role is None
+            and entity_group is None
         )
 
         if use_short_syntax:
-            return f"[{entity_text}]({entity_type})"
+            return f"({entity_type})"
         else:
             entity_dict = OrderedDict(
                 [
@@ -143,10 +191,49 @@ class TrainingDataWriter:
                 [(k, v) for k, v in entity_dict.items() if v is not None]
             )
 
-            return f"[{entity_text}]{json.dumps(entity_dict)}"
+            return f"{json.dumps(entity_dict)}"
+
+    @staticmethod
+    def generate_entity(
+        text: Text, entity: Union[Dict[Text, Any], List[Dict[Text, Any]]]
+    ) -> Text:
+        """Generates text for one or multiple entity objects.
+
+        Args:
+            text: The un-annotated text
+            entity: One or multiple entity annotations for one part of this text
+
+        Returns:
+            Annotated part of the text
+        """
+        if isinstance(entity, list):
+            entity_text = text[
+                entity[0][ENTITY_ATTRIBUTE_START] : entity[0][ENTITY_ATTRIBUTE_END]
+            ]
+            return (
+                f"[{entity_text}]["
+                + ", ".join(
+                    [
+                        TrainingDataWriter.generate_entity_attributes(
+                            text=entity_text, entity=e, short_allowed=False
+                        )
+                        for e in entity
+                    ]
+                )
+                + "]"
+            )
+        else:
+            entity_text = text[
+                entity[ENTITY_ATTRIBUTE_START] : entity[ENTITY_ATTRIBUTE_END]
+            ]
+            return f"[{entity_text}]" + TrainingDataWriter.generate_entity_attributes(
+                text=entity_text, entity=entity, short_allowed=True
+            )
 
 
 class JsonTrainingDataReader(TrainingDataReader):
+    """Training data reader specific for JSON files."""
+
     def reads(self, s: Text, **kwargs: Any) -> "TrainingData":
         """Transforms string into json object and passes it on."""
         js = json.loads(s)
